@@ -1,8 +1,8 @@
 if (getRversion() >= "3.1.0") {
   utils::globalVariables(c(
-    "distance", "i.size", "ind", "indClDT", "initialPixels", "n", "newQuantity",
-    "numNeighs", "numRetries", "origIndex", "pixels", "quantityAdj",
-    "quantityAdj2", "state", "size", "tooBigByNCells"
+    "distance", "from", "i.size", "ind", "indClDT", "initialPixels",
+    "n", "newQuantity", "numNeighs", "numRetries", "origIndex", "pixels",
+    "quantityAdj", "quantityAdj2", "state", "size", "tooBigByNCells", "V1", "proportion"
   ))
 }
 
@@ -188,6 +188,15 @@ if (getRversion() >= "3.1.0") {
 #' @param asymmetryAngle A numeric or \code{RasterLayer} indicating the angle in degrees
 #'                      (0 is "up", as in North on a map),
 #'                      that describes which way the \code{asymmetry} is.
+#' @param allowOverlap Logical. If TRUE, then individual events can overlap with one another,
+#'                     i.e., they do not interact (this is slower than if
+#'                     allowOverlap = FALSE). Default is FALSE. This can also be \code{NA},
+#'                     which means that the event can overlap with other events,
+#'                     and also itself. This would be, perhaps, useful for dispersal of,
+#'                     say, insect swarms.
+#'
+#' @param plot.it  If TRUE, then plot the raster at every iteraction,
+#'                   so one can watch the spread2 event grow.
 #'
 #' @inheritParams spread
 #'
@@ -304,36 +313,22 @@ if (getRversion() >= "3.1.0") {
 #'
 #' @aliases spread2
 #' @export
+#' @importFrom data.table copy
 #' @importFrom magrittr %>%
 #' @importFrom quickPlot Plot
 #' @name spread2
 #' @rdname spread2
 #'
-setGeneric(
-  "spread2",
+#' @example inst/examples/example_spread2.R
+#'
+spread2 <-
   function(landscape, start = ncell(landscape) / 2 - ncol(landscape) / 2,
            spreadProb = 0.23, asRaster = TRUE, maxSize, exactSize, directions = 8L,
            iterations = 1e6L, returnDistances = FALSE, returnFrom = FALSE,
            spreadProbRel = NA_real_, plot.it = FALSE, circle = FALSE,
            asymmetry = NA_real_, asymmetryAngle = NA_real_, allowOverlap = FALSE,
            neighProbs = NA_real_, skipChecks = FALSE) {
-  standardGeneric("spread2")
-})
 
-#' @param plot.it  If TRUE, then plot the raster at every iteraction,
-#'                   so one can watch the spread2 event grow.
-#'
-#' @rdname spread2
-#'
-#' @example inst/examples/example_spread2.R
-#'
-setMethod(
-  "spread2",
-  signature(landscape = "RasterLayer"),
-  definition = function(landscape, start, spreadProb, asRaster, maxSize, exactSize,
-                        directions, iterations, returnDistances, returnFrom,
-                        spreadProbRel, plot.it, circle, asymmetry, asymmetryAngle,
-                        allowOverlap, neighProbs, skipChecks) {
     #### assertions ###############
     assertClass(landscape, "Raster")
     ncells <- ncell(landscape)
@@ -401,7 +396,7 @@ setMethod(
     # Step 0 - set up objects -- main ones: dt, clusterDT -- get them from attributes
     ## on start or initiate them
     smallRaster <- ncells < 4e7 # should use bit vector (RAM) or ff raster (Disk)
-    canUseAvailable <- !allowOverlap
+    canUseAvailable <- !(isTRUE(allowOverlap) | is.na(allowOverlap))
     if (missing(maxSize)) {
       maxSize <- NA
     }
@@ -414,6 +409,7 @@ setMethod(
 
     # returnDistances = TRUE and circle = TRUE both require distance calculations
     needDistance <- returnDistances | circle
+    usingAsymmetry <- !is.na(asymmetry)
 
     # This means that if an event can not spread any more, it will try 10 times, incl. 2 jumps
     maxRetriesPerID <- 10
@@ -464,6 +460,10 @@ setMethod(
 
       setkeyv(clusterDT, "initialPixels")
       if (needDistance) set(dt, , "distance", 0) # it is zero distance to self
+      if (usingAsymmetry) {
+        set(dt, , "effectiveDistance", 0) # it is zero distance to self
+        set(dt, , "distClass", 0) # it is zero distance to self
+      }
       totalIterations <- 0
 
     } else {
@@ -513,7 +513,8 @@ setMethod(
     }
 
     whTooSmall <- integer()
-    dtPotentialColNames <- c("id", "from", "to", "state", "distance"[needDistance])
+    dtPotentialColNames <- c("id", "from", "to", "state", "distance"[needDistance],
+                             "effectiveDistance"[usingAsymmetry])
 
     # start at iteration 0, note: totalIterations is also maintained,
     # which persists during iterative calls to spread2
@@ -602,7 +603,7 @@ setMethod(
         fromPts <- xyFromCell(landscape, dtPotential$id)
         toPts <- xyFromCell(landscape, dtPotential$to)
         dists <- pointDistance(p1 = fromPts, p2 = toPts, lonlat = FALSE)
-        if (!is.na(asymmetry)) {
+        if (usingAsymmetry) {
           actualAsymmetry <- if (length(asymmetry) == 1) {
             asymmetry
           } else {
@@ -614,16 +615,64 @@ setMethod(
             asymmetryAngle[dtPotential$to]
           }
 
-          angleQualities <- angleQuality(dtPotential, landscape, actualAsymmetryAngle)
+          angleQualities <- angleQuality(from = dtPotential$id, to = dtPotential$to,
+                                         landscape, actualAsymmetryAngle)
           naAQ <- is.na(angleQualities[, "angleQuality"])
           angleQualities[naAQ, "angleQuality"] <- 1
           # convert dists to effective distance
           effDists <- dists * ((2 - angleQualities[, "angleQuality"]) / 2 *
                                  (actualAsymmetry - 1) + 1)
+
+          # For asymmetry, we also may want to know what proportion of the outward spreading
+          #  event will hit each pixel, not just the effectiveDistance
+          lociHere <- if (is.numeric(start)) start else
+            attributes(dt)$spreadState$clusterDT$initialPixels
+          # pureCircle <- cir(landscape,
+          #                   loci = lociHere,
+          #                   allowOverlap = TRUE, allowDuplicates = FALSE,
+          #                   maxRadius = totalIterations,
+          #                   minRadius = totalIterations - 0.999999,
+          #                   returnIndices = TRUE,
+          #                   returnDistances = TRUE,
+          #                   includeBehavior = "excludePixels")
+
+
+          # This is a very fast version with allowOverlap = TRUE, allowDuplicates = FALSE,
+          #   returnIndices = TRUE, returnDistancse = TRUE, and includeBehaviour = "excludePixels"
+          pureCircle <- .cirSpecialQuick(landscape,
+                            loci = lociHere,
+                            maxRadius = totalIterations,
+                            minRadius = totalIterations - 0.999999)
+          pureCircle <- cbind(pureCircle[, c("id", "indices", "dists"), drop = FALSE],
+                              distClass = ceiling(pureCircle[, "dists"]))
+          colnames(pureCircle)[2] <- c("to")
+
+          theoreticalAngleQualities <- angleQuality(pureCircle[, "id", drop = FALSE],
+                                                    pureCircle[, "to", drop = FALSE],
+                                                    landscape,
+                                                    actualAsymmetryAngle = actualAsymmetryAngle)
+          naAQ <- is.na(theoreticalAngleQualities[, "angleQuality"])
+          theoreticalAngleQualities[naAQ, "angleQuality"] <- 1
+          # convert dists to effective distance
+          effDists1 <- pureCircle[, "dists"] *
+            ((2 - theoreticalAngleQualities[, "angleQuality"]) / 2 * (actualAsymmetry - 1) + 1)
+
+          pc <- pureCircle[, "dists"] / effDists1
+          pureCircle <- cbind(pureCircle, proportion = pc)
+          pureCircle <- as.data.table(pureCircle)
+          pureCircle[, proportion := proportion / sum(proportion), by = "id"]
+          set(pureCircle, , "dists", NULL)
+          setkeyv(pureCircle, c("id", "to"))
+          pureCirclePrev <- attr(dt, "spreadState")$pureCircle
+          if (!is.null(pureCirclePrev)) {
+            pureCircle <- rbindlist(list(pureCircle, pureCirclePrev),
+                                    use.names = FALSE, fill = FALSE)
+            #pureCircle <- unique(pureCircle)
+          }
         }
 
         if (circle) {
-          if (!is.na(asymmetry)) {
+          if (usingAsymmetry) {
             distKeepers <- effDists %<=% totalIterations & effDists %>>%
               (totalIterations - 1)
             dtPotentialAsymmetry <- dtPotential[!distKeepers]
@@ -642,12 +691,19 @@ setMethod(
             distKeepers <- dists %<=% totalIterations & dists %>>%
               (totalIterations - 1)
           }
+
+          dtPotentialAllNeighs <- copy(dtPotential)
+          setkeyv(dtPotentialAllNeighs, "from")
           dtPotential <- dtPotential[distKeepers]
           dists <- dists[distKeepers]
         }
         set(dtPotential, , "distance", dists)
-        if (!is.na(asymmetry)) {
+        if (usingAsymmetry) {
           set(dtPotential, , "effectiveDistance", effDists[distKeepers])
+          if (circle) {
+            dtPotential <- dtPotential[pureCircle, nomatch = 0, on = c("id", "to")][
+              , proportion := proportion / .N, by = c("id", "to")]
+          }
         }
       }
 
@@ -728,7 +784,7 @@ setMethod(
       # Step 6a -- asymmetry -- this will modify spreadProb if it is not a circle
       #  -- circle asymmetry happens elsewhere
       # modify actualSpreadProb if there is asymmetry
-      if (!is.na(asymmetry) & !circle) {
+      if (usingAsymmetry & !circle) {
         actualAsymmetry <- if (length(asymmetry) == 1) {
           asymmetry
         } else {
@@ -740,7 +796,8 @@ setMethod(
           asymmetryAngle[dtPotential$to]
         }
 
-        angleQualities <- angleQuality(dtPotential, landscape, actualAsymmetryAngle)
+        angleQualities <- angleQuality(from = dtPotential$id, to = dtPotential$to,
+                                       landscape, actualAsymmetryAngle)
 
         naAQ <- is.na(angleQualities[, "angleQuality"])
         angleQualities[naAQ, "angleQuality"] <- actualSpreadProb[naAQ]
@@ -752,16 +809,19 @@ setMethod(
 
       # Step 7 - Remove duplicates & bind dt and dtPotential
       if (anyNA(neighProbs)) {
-        if (allowOverlap | !canUseAvailable) {
+        if (isTRUE(allowOverlap) | is.na(allowOverlap) | !canUseAvailable) {
           # overlapping allowed
           dtPotential <- dtPotential[spreadProbSuccess]
           dtNROW <- NROW(dt)
           dt <- rbindlistDtDtpot(dt, dtPotential, returnFrom, needDistance, dtPotentialColNames)
 
-          dt[, `:=`(dups = duplicatedInt(pixels)), by = initialPixels]
-          dupes <- dt$dups
-          set(dt, , "dups", NULL)
-          dt <- dt[!dupes]
+          # this is to prevent overlap within an event... in some cases, overlap within event is desired, so skip this block
+          if (!is.na(allowOverlap)) {
+            dt[, `:=`(dups = duplicatedInt(pixels)), by = initialPixels]
+            dupes <- dt$dups
+            set(dt, , "dups", NULL)
+            dt <- dt[!dupes]
+          }
 
           # remove all the duplicated ones from dtPotential
           dtPotential <- dt[-seq_len(dtNROW)]
@@ -783,6 +843,13 @@ setMethod(
           dtPotential <- dtPotential[whSuccNoDupsCurItAndAll]
 
           dt <- rbindlistDtDtpot(dt, dtPotential, returnFrom, needDistance, dtPotentialColNames)
+
+          if (circle) {
+            if (usingAsymmetry) {
+              saturated <- dtPotentialAllNeighs[, sum(to %in% dt$pixels) == directions,
+                                                by = from][V1 == TRUE]$from
+            }
+          }
         }
       } else {
         # neighProbs -- duplication checking already happened, but
@@ -856,6 +923,16 @@ setMethod(
       } # end size-based assessments
 
       # Step 9 # Change states of cells
+      if (usingAsymmetry){
+        if(!(isTRUE(allowOverlap) | is.na(allowOverlap))) {
+          if (circle) {
+            if(length(saturated)) {
+              set(dt, which(dt$pixels %in% saturated), "state", "activeSource")
+            }
+          }
+        }
+      }
+
       notInactive <- dt$state != "inactive" # currently activeSource, successful, or holding
       whNotInactive <- which(notInactive)
       activeStates <- dt$state[whNotInactive]
@@ -901,6 +978,10 @@ setMethod(
     if (canUseAvailable) {
       attrList <- append(attrList, list(notAvailable = notAvailable))
     }
+    if (usingAsymmetry) {
+      if(exists("pureCircle", inherits = FALSE))
+        attrList <- append(attrList, list(pureCircle = pureCircle))
+    }
     setattr(dt, "spreadState", attrList)
 
     # Step 12 -- return either raster or data.table
@@ -913,7 +994,7 @@ setMethod(
     }
     return(dt)
   }
-)
+
 
 #' Internal helper
 #'
@@ -965,17 +1046,18 @@ reorderColsWDistance <- function(needDistance, dtPotential, dtPotentialColNames)
     )
 }
 
-#' @param dtPotential \code{data.table} of potential spread locations.
+#' @param from vector of cell locations which are the "from" or starting cells
+#' @param to vector of same length as \code{from} which are the "to" or receiving cells
 #' @param landscape \code{RasterLayer} passed from \code{spread2}.
 #' @param actualAsymmetryAngle Angle in degrees, either a vector length 1 or
 #'                             vector \code{NROW(dtPotential)}.
 #' @keywords internal
 #' @rdname spread2-internals
 #'
-angleQuality <- function(dtPotential, landscape, actualAsymmetryAngle) {
-  from <- cbind(id = dtPotential$id, xyFromCell(landscape, dtPotential$id))
-  to <- cbind(id = dtPotential$id, xyFromCell(landscape, dtPotential$to))
-  d <- .pointDirection(from = from, to = to)
+angleQuality <- function(from, to, landscape, actualAsymmetryAngle) {
+  from1 <- cbind(id = from, xyFromCell(landscape, from))
+  to1 <- cbind(id = from, xyFromCell(landscape, to))
+  d <- .pointDirection(from = from1, to = to1)
 
   angleQuality <- cbind(angleQuality = (cos(d[, "angles"] - rad(actualAsymmetryAngle)) + 1), d)
   angleQuality
